Module executable

Module executable 

InterchainTokenExecutable

This is an executable interface that accepts an interchain token from ITS contract along with an arbitrary message.

This is similar to the AxelarExecutableInterface but meant for messages sent with an ITS token.

Structs

InterchainTokenExecutableClient
InterchainTokenExecutableClient is a client for calling the contract defined in “InterchainTokenExecutableInterface”.

Traits

CustomInterchainTokenExecutable
This trait must be implemented by a contract to be compatible with the InterchainTokenExecutableInterface.
InterchainTokenExecutableInterface
Interface for an Interchain Token Executable app. Use the InterchainTokenExecutable derive macro to implement this interface.

Derive Macros

InterchainTokenExecutable