pub struct InterchainTokenExecutableClient<'a> {
pub env: Env,
pub address: Address,
/* private fields */
}Expand description
InterchainTokenExecutableClient is a client for calling the contract defined in “InterchainTokenExecutableInterface”.
Fields
env: Envaddress: AddressImplementations
Sourceimpl<'a> InterchainTokenExecutableClient<'a>
impl<'a> InterchainTokenExecutableClient<'a>
Sourceimpl<'a> InterchainTokenExecutableClient<'a>
impl<'a> InterchainTokenExecutableClient<'a>
Sourcepub fn interchain_token_service(&self) -> Address
pub fn interchain_token_service(&self) -> Address
Returns the address of the interchain token service contract that is authorized to execute arbitrary payloads on this contract
Sourcepub fn try_interchain_token_service(
&self,
) -> Result<Result<Address, <Address as TryFromVal<Env, Val>>::Error>, Result<Error, InvokeError>>
pub fn try_interchain_token_service( &self, ) -> Result<Result<Address, <Address as TryFromVal<Env, Val>>::Error>, Result<Error, InvokeError>>
Returns the address of the interchain token service contract that is authorized to execute arbitrary payloads on this contract
Sourcepub fn execute_with_interchain_token(
&self,
source_chain: &String,
message_id: &String,
source_address: &Bytes,
payload: &Bytes,
token_id: &BytesN<32>,
token_address: &Address,
amount: &i128,
)
pub fn execute_with_interchain_token( &self, source_chain: &String, message_id: &String, source_address: &Bytes, payload: &Bytes, token_id: &BytesN<32>, token_address: &Address, amount: &i128, )
Execute a cross-chain message with the given payload and token.
Authorization
- Only callable by ITS contract.
Sourcepub fn try_execute_with_interchain_token(
&self,
source_chain: &String,
message_id: &String,
source_address: &Bytes,
payload: &Bytes,
token_id: &BytesN<32>,
token_address: &Address,
amount: &i128,
) -> Result<Result<(), <() as TryFromVal<Env, Val>>::Error>, Result<Error, InvokeError>>
pub fn try_execute_with_interchain_token( &self, source_chain: &String, message_id: &String, source_address: &Bytes, payload: &Bytes, token_id: &BytesN<32>, token_address: &Address, amount: &i128, ) -> Result<Result<(), <() as TryFromVal<Env, Val>>::Error>, Result<Error, InvokeError>>
Execute a cross-chain message with the given payload and token.
Authorization
- Only callable by ITS contract.