InterchainTokenExecutable
This is an executable interface that accepts an interchain token from ITS contract along with an arbitrary message.
This is similar to the AxelarExecutableInterface but meant for messages sent with an ITS token.
Structs
- Interchain
Token Executable Client - InterchainTokenExecutableClient is a client for calling the contract defined in “InterchainTokenExecutableInterface”.
Traits
- Custom
Interchain Token Executable - This trait must be implemented by a contract to be compatible with the
InterchainTokenExecutableInterface. - Interchain
Token Executable Interface - Interface for an Interchain Token Executable app. Use the
InterchainTokenExecutablederive macro to implement this interface.